<?php
/* Copyright (c) 1998-2013 ILIAS open source, Extended GPL, see docs/LICENSE */

/**
 * Class ilObjQuestionSetPoolGUI
 * Date: 23.09.13
 * Time: 11:09
 * @author            Thomas Joußen <tjoussen@databay.de>
 * @ilCtrl_isCalledBy ilObjQuestionSetPoolGUI: ilRepositoryGUI, ilAdministrationGUI, ilObjPluginDispatchGUI
 * @ilCtrl_Calls      ilObjQuestionSetPoolGUI: ilPermissionGUI, ilInfoScreenGUI, ilObjectCopyGUI, ilCommonActionDispatcherGUI, ilMDEditorGUI, ilPublicUserProfileGUI
 * @ilCtrl_Calls      ilObjQuestionSetPoolGUI: ilAssQuestionPageGUI
 * @ilCtrl_Calls      ilObjQuestionSetPoolGUI: ilQuestionSetPoolSkillAdministrationGUI, ilQuestionSetPoolExportGUI
 */
class ilObjQuestionSetPoolGUI extends ilObjectPluginGUI implements ilDesktopItemHandling
{

	/**
	 * @var ilObjQuestionSetPool
	 */
	public $object;

	public $plugin;
	
	/**
	 * @param string $cmd
	 * @return mixed
	 */
	public function performCommand($cmd)
	{
		/**
		 * @var $ilTabs ilTabsGUI
		 * @var $tpl    ilTemplate
		 */
		global $ilTabs, $tpl;

		$tpl->setDescription($this->object->getDescription());

		$this->lng->loadLanguageModule('assessment');
		$tpl->addCss('./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/templates/default/xqsp.css');

		$next_class = $this->ctrl->getNextClass($this);
		switch($next_class)
		{
			case 'ilquestionsetpoolskilladministrationgui':
				require_once 'controllers/class.ilQuestionSetPoolSkillAdministrationGUI.php';
				$this->prepareOutput();
				$this->addHeaderAction();

				$gui = new ilQuestionSetPoolSkillAdministrationGUI($this);
				$this->ctrl->forwardCommand($gui);
				break;

			case 'ilmdeditorgui':
				$this->checkPermission('write');
				$md_gui = new ilMDEditorGUI($this->object->getId(), 0, $this->object->getType());
				$md_gui->addObserver($this->object, 'MDUpdateListener', 'General');
				$ilTabs->setTabActive('meta_data');
				return $this->ctrl->forwardCommand($md_gui);
				break;

			case 'ilpublicuserprofilegui':
				$profile_gui = new ilPublicUserProfileGUI($_GET['user']);
				$profile_gui->setBackUrl($this->ctrl->getLinkTarget($this, 'showContent'));
				$this->tpl->setContent($this->ctrl->forwardCommand($profile_gui));
				break;

			case 'ilcommonactiondispatchergui':
				$gui = ilCommonActionDispatcherGUI::getInstanceFromAjaxCall();
				$this->ctrl->forwardCommand($gui);
				break;
			case "ilquestionsetpoolexportgui":
				$this->checkPermission('write');
				$ilTabs->setTabActive('export');
				$this->getPlugin()->includeClass('class.ilQuestionSetPoolExportGUI.php');
				$exp_gui = new ilQuestionSetPoolExportGUI($this);
				$exp_gui->addFormat('xml', $this->lng->txt('qpl_export_xml'));
				$this->ctrl->forwardCommand($exp_gui);
				break;
			default:
				switch($cmd)
				{
					case 'redrawHeaderAction':
					case 'addToDesk':
					case 'removeFromDesk':
						if(in_array($cmd, array('addToDesk', 'removeFromDesk')))
						{
							$cmd .= 'Object';
						}
						$this->checkPermission('read');
						$this->$cmd();
						break;

					default:
						$this->ensureEntityExistance();
                        require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/dispatcher/class.ilPluginDispatcher.php";
                        $dispatcher = ilPluginDispatcher::getInstance($this);
						$dispatcher->dispatch($cmd);
						break;
				}
				break;
		}

		$this->addHeaderAction();
	}

	protected function redirectToDefaultPage()
	{
		$this->ctrl->redirect($this);
	}
	
	protected function ensureEntityExistance()
	{
		if(isset($_GET['set_id']) && (int)$_GET['set_id'])
		{
			$set_id = (int)$_GET['set_id'];
			$this->getPlugin()->includeClass('models/class.ilQuestionSetPoolSet.php');
			if(!ilQuestionSetPoolSet::exists($set_id))
			{
				ilUtil::sendFailure($this->txt('poss_sync_err_set'), true);
				$this->redirectToDefaultPage();
			}
		}

		if(isset($_GET['path_id']) && (int)$_GET['path_id'])
		{
			$path_id = (int)$_GET['path_id'];
			$this->getPlugin()->includeClass('models/class.ilQuestionSetPoolPath.php');
			if(!ilQuestionSetPoolPath::exists($path_id))
			{
				ilUtil::sendFailure($this->txt('poss_sync_err_path'), true);
				$this->redirectToDefaultPage();
			}
		}

		if(isset($_GET['node_id']) && (int)$_GET['node_id'])
		{
			$node_id = (int)$_GET['node_id'];
			$this->getPlugin()->includeClass('models/class.ilQuestionSetPoolNode.php');
			if(!ilQuestionSetPoolNode::exists($node_id))
			{
				ilUtil::sendFailure($this->txt('poss_sync_err_node'), true);
				$this->redirectToDefaultPage();
			}
		}
	}

	/**
	 * Public wrapper for permission checks
	 * @param string $permission
	 * @return bool
	 */
	public function hasPermission($permission)
	{
		return $this->checkPermissionBool($permission);
	}

	/**
	 * Public wrapper for permission assumption
	 * @param string $permission
	 * @return bool
	 */
	public function ensurePermission($permission)
	{
		return $this->checkPermission($permission);
	}

	/**
	 * 
	 */
	public function populateContentStyleBlock()
	{
		include_once("./Services/Style/Content/classes/class.ilObjStyleSheet.php");
		$this->tpl->setCurrentBlock( "ContentStyle" );
		$this->tpl->setVariable( "LOCATION_CONTENT_STYLESHEET",ilObjStyleSheet::getContentStylePath(0));
		$this->tpl->parseCurrentBlock();

		$this->tpl->addCss(ilUtil::getStyleSheetLocation("output", "test_print.css", "Modules/Test"), "print");
		$this->tpl->addCss(ilUtil::getStyleSheetLocation("output", "ta.css", "Modules/Test"), "screen");
	}

	/**
	 * 
	 */
	public function populateSyntaxStyleBlock()
	{
		$this->tpl->setCurrentBlock( "SyntaxStyle" );
		$this->tpl->setVariable( "LOCATION_SYNTAX_STYLESHEET",ilObjStyleSheet::getSyntaxStylePath());
		$this->tpl->parseCurrentBlock();
	}

	/**
	 * Functions that must be overwritten
	 */
	public function getType()
	{
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/class.ilQuestionSetPoolPlugin.php";
		return ilQuestionSetPoolPlugin::getPluginId();
	}

	/**
	 * Cmd that will be redirected to after creation of a new object.
	 */
	public function getAfterCreationCmd()
	{
		return "ilQuestionSetPoolPropertiesGUI.edit";
	}

	public function getStandardCmd()
	{
		return "ilQuestionSetPoolSetsGUI.show";
	}
	
	/**
	 * @param string $type
	 * @return array
	 */
	protected function initCreationForms($type)
	{
		return array(
			self::CFORM_NEW => $this->initCreateForm($type),
			self::CFORM_IMPORT => $this->initImportForm($type)
		);
	}

	/**
	 * @param string $type
	 * @return ilPropertyFormGUI
	 */
	public function  initCreateForm($type)
	{
		$form = parent::initCreateForm($type);
		// Add additional form elements for creation form here
		return $form;
	}
	
	/**
	 * @param $a_new_type
	 * @return ilPropertyFormGUI
	 */
	public function initImportForm($a_new_type)
	{
		$form =  parent::initImportForm($a_new_type); 
		
		// select question pool
		include_once("./Modules/Test/classes/class.ilObjTest.php");
		$tst = new ilObjTest();
		$questionpools = $tst->getAvailableQuestionpools(TRUE, FALSE, TRUE, TRUE);
		$this->getPlugin()->includeClass('class.ilQuestionSetPoolImporter.php');
		$options = array(
			ilQuestionSetPoolImporter::SINGLE_IMPORT_NEW_POOL => $this->lng->txt("assessment_new_pool"),
			ilQuestionSetPoolImporter::SINGLE_IMPORT_NO_POOL  => $this->lng->txt('assessment_no_pool')
		);

		if (is_array($questionpools) && count($questionpools))
		{
			foreach ($questionpools as $key => $value)
			{
				$options[$key] = $value["title"];
			}
		}
		$pool = new ilSelectInputGUI($this->lng->txt("select_questionpool"), "qpl");
		$pool->setOptions($options);
		$form->addItem($pool);
		
		return $form;
	}
	
	/**
	 *
	 */
	public function setTabs()
	{
		$this->tabs_gui->clearTargets();

		if($this->checkPermissionBool('read'))
		{
			$this->tabs_gui->addTab('content', $this->txt('question_sets'), $this->ctrl->getLinkTarget($this, 'ilQuestionSetPoolSetsGUI.show'));
		}

		$this->addInfoTab();

		if($this->checkPermissionBool('read') && $this->object->isSkillServiceEnabled() && ilObjQuestionSetPool::isSkillManagementGloballyActivated())
		{
			$link = $this->ctrl->getLinkTargetByClass(
				array('ilQuestionSetPoolSkillAdministrationGUI', 'ilAssQuestionSkillAssignmentsGUI'),
				ilAssQuestionSkillAssignmentsGUI::CMD_SHOW_SKILL_QUEST_ASSIGNS
			);

			$this->tabs_gui->addTarget('tst_tab_competences', $link);
		}

		if($this->checkPermissionBool('write'))
		{
			$this->tabs_gui->addTab('properties', $this->lng->txt('properties'), $this->ctrl->getLinkTarget($this, 'ilQuestionSetPoolPropertiesGUI.edit'));
			$this->tabs_gui->addTarget('meta_data', $this->ctrl->getLinkTargetByClass('ilmdeditorgui', ''), '', 'ilmdeditorgui');
			$this->tabs_gui->addTarget("export", $this->ctrl->getLinkTargetByClass("ilquestionsetpoolexportgui", ""), "", "ilquestionsetpoolexportgui");
		}

		$this->addPermissionTab();
	}

	/**
	 * @param string $a_sub_type
	 * @param int    $a_sub_id
	 * @return ilObjectListGUI|ilObjQuestionSetPoolListGUI
	 */
	protected function initHeaderAction($a_sub_type = null, $a_sub_id = null)
	{
		/**
		 * @var $ilUser ilObjUser
		 */
		global $ilUser;

		$lg = parent::initHeaderAction();
		if($lg instanceof ilObjQuestionSetPoolListGUI)
		{
			if($ilUser->getId() != ANONYMOUS_USER_ID)
			{
				// Maybe handle notifications in future ...
			}
		}

		return $lg;
	}
}
