<?php
/* Copyright (c) 1998-2016 ILIAS open source, Extended GPL, see docs/LICENSE */

/**
 * Class ilObjQuestionSetPoolXMLParser
 * @author Nadia Matuschek <nmatuschek@databay.de>
 */
class ilObjQuestionSetPoolXMLParser extends ilSaxParser
{
	/**
	 * @var ilObjQuestionSetPool
	 */
	private $xqsp_obj;
	
	private $inSettingsTag;
	
	private $inMetaDataTag;
	private $inMdGeneralTag;
	
	public $sets = array();
	
	private $questions = array();
	private $paths = array();
	private $nodes = array();
	private $jump_conditions = array();
	protected $importDirectory = '';
	
	/**
	 * @param ilObjQuestionSetPool $xqsp_obj
	 * @param                      $xmlFile
	 */
	public function __construct(ilObjQuestionSetPool $xqsp_obj, $xmlFile)
	{
		$this->xqsp_obj = $xqsp_obj;
		
		$this->inSettingsTag  = false;
		$this->inMetaDataTag  = false;
		$this->inMdGeneralTag = false;
		
		parent::__construct($xmlFile);
	}
	
	/**
	 * Set import directory
	 *
	 * @param	string	import directory
	 */
	public function setImportDirectory($a_val)
	{
		$this->importDirectory = $a_val;
	}
	
	/**
	 * Get import directory
	 *
	 * @return	string	import directory
	 */
	public function getImportDirectory()
	{
		return $this->importDirectory;
	}
	
	/**
	 * @param $xmlParser
	 */
	public function setHandlers($xmlParser)
	{
		xml_set_object($xmlParser, $this);
		xml_set_element_handler($xmlParser, 'handlerBeginTag', 'handlerEndTag');
		xml_set_character_data_handler($xmlParser, 'handlerCharacterData');
	}
	
	/**
	 * @param $xmlParser
	 * @param $tagName
	 * @param $tagAttributes
	 */
	public function handlerBeginTag($xmlParser, $tagName, $tagAttributes)
	{
		switch($tagName)
		{
			case 'MetaData':
				$this->inMetaDataTag = true;
				break;
			
			case 'General':
				if($this->inMetaDataTag)
				{
					$this->inMdGeneralTag = true;
				}
				break;
			
			case 'Description':
				if($this->inMetaDataTag && $this->inMdGeneralTag)
				{
					$this->cdata = '';
				}
				break;
			
			case 'Settings':
				$this->inSettingsTag = true;
				break;
			
			case 'Online':
			case 'SkillService':
				if($this->inSettingsTag)
				{
					$this->cdata = '';
				}
				break;
			case 'SetId':
			case 'SetTitle':
			case 'SetDescription':
			case 'OriginalId':
				$this->cdata = '';
				break;
		}
	}
	
	/**
	 * @param $xmlParser
	 * @param $tagName
	 */
	public function handlerEndTag($xmlParser, $tagName)
	{
		switch($tagName)
		{
			case 'MetaData':
				$this->inMetaDataTag = false;
				break;
			
			case 'General':
				if($this->inMetaDataTag)
				{
					$this->inMdGeneralTag = false;
				}
				break;
			
			case 'Title':
				if($this->inMetaDataTag && $this->inMdGeneralTag)
				{
					$this->xqsp_obj->setTitle(trim($this->cdata));
					$this->cdata = '';
				}
				break;
			
			case 'Description':
				if($this->inMetaDataTag && $this->inMdGeneralTag)
				{
					$this->xqsp_obj->setDescription(trim($this->cdata));
					$this->cdata = '';
				}
				break;
			
			case 'Settings':
				$this->inSettingsTag = false;
				break;
			
			case 'Online':
				$this->xqsp_obj->setOnline((bool)trim($this->cdata));
				$this->cdata = '';
				break;
			
			case 'SkillService':
				$this->xqsp_obj->setSkillServiceEnabled((bool)trim($this->cdata));
				$this->cdata = '';
				break;
			// sets	
			case 'SetId':
			case 'SetTitle':
			case 'SetDescription':
			case 'OriginalId':
				$this->import_set[$tagName] = trim($this->cdata);
				$this->cdata                = '';
				break;
			// question
			case 'QuestionId':
			case 'QuestionIndex':
			case 'QuestionOriginalId':
				$this->import_question[$tagName] = trim($this->cdata);
				$this->cdata                     = '';
				break;
			// paths
			case 'PathId':
			case 'PathTitle':
			case 'PathQuestionSetId':
			case 'PathIndex':
				$this->import_path[$tagName] = trim($this->cdata);
				$this->cdata                 = '';
				break;
			//nodes
			case 'NodeId':
			case 'NodeQuestionId':
			case 'NodePathId':
			case 'NodeIndex':
			case 'NodeInitialCondition':
			case 'NodeDominantScoring':
			case 'NodeTrueFeedback':
			case 'NodeFalseFeedback':
				$this->import_node[$tagName] = trim($this->cdata);
				$this->cdata                 = '';
				break;
			// jump conditions 
			case 'JumpConditionId':
			case 'JumpConditionText':
			case 'JumpConditionNodeId':
			case 'JumpConditionQuestionId':
			case 'JumpConditionRepetitions':
			case 'JumpConditionSorting':
			case 'JumpConditionTrueFeedback':
			case 'JumpConditionTrueJumpFeedback':
			case 'JumpConditionFalseFeedback':
				$this->import_jump_condition[$tagName] = trim($this->cdata);
				$this->cdata                           = '';
				break;
			case 'Set':
				//'addSet';
				$this->sets[]     = array($this->import_set, 'questions' => $this->questions, 'paths' => $this->paths);
				$this->questions  = array();
				$this->paths      = array();
				$this->import_set = array();
				break;
			
			case 'Question':
				// addQuesion
				$this->questions[]     = $this->import_question;
				$this->import_question = array();
				break;
			
			case 'Path':
				// addPath
				$this->paths[]     = array($this->import_path, 'nodes' => $this->nodes);
				$this->nodes       = array();
				$this->import_path = array();
				break;
			
			case 'Node':
				// addNode
				$this->nodes[]         = array($this->import_node, 'jump_conditions' => $this->jump_conditions);
				$this->jump_conditions = array();
				$this->import_node     = array();
				break;
			
			case 'JumpCondition':
				// addJumpConditition
				$this->jump_conditions[]     = $this->import_jump_condition;
				$this->import_jump_condition = array();
				break;
		}
	}
	
	public function handlerCharacterData($xmlParser, $charData)
	{
		if($charData != "\n")
		{
			// Replace multiple tabs with one space
			$charData = preg_replace("/\t+/", " ", $charData);
			
			$this->cdata .= $charData;
		}
	}
	
	public function getQuestionSetPoolObject()
	{
		return $this->xqsp_obj;
	}
	
	public function getSetsArray()
	{
		return $this->sets;
	}
}