<?php

require_once "abstract.ilPluginControllerFormGUI.php";

/**
 * Class ilQuestionSetPoolPathGUI
 *
 * Date: 21.11.13
 * Time: 10:22
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolPathGUI extends ilPluginControllerFormGUI
{

	protected $redirectCmd = "ilQuestionSetPoolNodesGUI.listNodes";

	public function edit()
	{
		$this->redirectIfNoQuestionsInSet();

		$model = $this->loadModel();
		$this->initForm($model);
		$this->bindModelToForm($model);
		$this->tpl->setContent($this->form->getHTML());
	}

	public function update()
	{
		$this->redirectIfNoQuestionsInSet();

		$model = $this->loadModel();
		$this->initForm($model);
		if($this->form->checkInput())
		{
			$model->bindForm($this->form);
			$model->update();
			ilUtil::sendSuccess($this->plugin->txt("msg_path_modified"), true);

			$this->redirect(
				$this->getAfterUpdateRedirectCmd(),
				$this->getAfterUpdateRedirectParams($model)
			);
		}

		$this->form->setValuesByPost();
		$this->tpl->setContent($this->form->getHTML());
	}

	public function updateAndNewNode()
	{
		$this->redirectCmd = "ilQuestionSetPoolNodeGUI.edit";
		$this->update();
	}


	/**
	 * Loads and returns the Model for the controller gui
	 * @return ilPluginModel
	 */
	protected function loadModel()
	{
		$model = new ilQuestionSetPoolPath((int)$_GET["path_id"]);
		$model->read();

		return $model;
	}

	/**
	 * Initialize the PropertyFormGUI for this controller
	 *
	 * @param ilQuestionSetPoolPath $model
	 */
	protected function initForm($model)
	{
		$this->form = new ilPropertyFormGUI();
		$this->form->setTitle(
			sprintf(
				$this->plugin->txt("path_title"),
				$model->getSet()->getTitle()
			)
		);

		$title = new ilTextInputGUI($this->plugin->txt("title"), "title");
		$title->setRequired(true);
		$this->form->addItem($title);

		if($model->getId() <= 0)
		{
			$start_question = new ilSelectInputGUI($this->plugin->txt('start_question'), "start_question");
			$start_question->setRequired(true);
			$start_question->setOptions($this->getStartQuestionOptions($model));
			$this->form->addItem($start_question);
		}

		$this->ctrl->setParameter($this->controller, "path_id", (int)$model->getId());
		$this->ctrl->setParameter($this->controller, "set_id", (int)$_GET['set_id']);
		$this->form->setFormAction($this->ctrl->getFormAction($this->controller));
		$this->ctrl->setParameter($this->controller, "set_id", "");
		$this->ctrl->setParameter($this->controller, "path_id", "");
		$this->form->addCommandButton("ilQuestionSetPoolPathGUI.updateAndNewNode", $this->plugin->txt("save_and_node"));
		$this->form->addCommandButton("ilQuestionSetPoolPathGUI.update", $this->plugin->txt("save"));
		$this->form->addCommandButton("ilQuestionSetPoolPathsGUI.listPaths", $this->plugin->txt("cancel"));
	}

	/**
	 * @param ilQuestionSetPoolPath $model
	 *
	 * @return array
	 */
	protected function getStartQuestionOptions($model)
	{
		$options   = array();
		$questions = $this->getExistingStartQuestion($model);

		if($questions == null)
		{
			$set = new ilQuestionSetPoolSet((int)$_GET["set_id"]);
			$set->read();
			$questions = $set->getQuestionList()->getData();
		}else{
			ilUtil::sendInfo($this->plugin->txt("start_question_defined"), false);
		}

		foreach ($questions as $question)
		{
			$options[$question['question_id']] = $question['title'];
		}

		return $options;
	}

	/**
	 * @param ilQuestionSetPoolPath $model
	 *
	 * @return array|null
	 */
	public function getExistingStartQuestion($model)
	{
		global $DIC;
		$ilDB = $DIC->database();

		$ilDB->setLimit(1, 0);
		$result = $ilDB->queryF(
			"SELECT q.question_id, q.title
			FROM qpl_questions q
			INNER JOIN rep_robj_xqsp_node n ON n.question_fi = q.question_id
			INNER JOIN rep_robj_xqsp_qs_qst qst ON qst.question_fi = q.question_id
			WHERE n.node_index = %s AND qst.questionset_fi = %s",
			array("integer", "integer"),
			array(1, $model->getSet()->getId())
		);

		$data = $ilDB->fetchAssoc($result);

		if($data != null)
		{
			$data = array($data);
		}

		return $data;
	}

	/**
	 * @param ilQuestionSetPoolSet $set
	 * @return bool
	 */
	protected function doesSetHaveQuestions($set)
	{
		return is_array($set->getQuestionList()->getData()) ?  count($set->getQuestionList()->getData()) > 0 :  0;
	}

	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return "content";
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		return array(
			'models/class.ilQuestionSetPoolPath.php',
			"models/class.ilQuestionSetPoolSet.php"
		);
	}

	/**
	 * Get the redirect action after an update was successful
	 * @return string
	 */
	protected function getAfterUpdateRedirectCmd()
	{
		return $this->redirectCmd;
	}

	/**
	 * @param ilQuestionSetPoolPath $model
	 *
	 * @return array
	 */
	protected function getAfterUpdateRedirectParams($model = NULL)
	{
		return array(
			"set_id" => (int)$_GET['set_id'],
			"path_id" => ($model == NULL ? NULL : (int)$model->getId()) 
		);
	}

	protected function redirectIfNoQuestionsInSet()
	{
		$set = new ilQuestionSetPoolSet((int)$_GET["set_id"]);
		$set->read();

		if (!$this->doesSetHaveQuestions($set))
		{
			ilUtil::sendFailure($this->plugin->txt("no_questions_in_set"), true);
			$this->redirect(
				 $this->getAfterUpdateRedirectCmd(),
					 $this->getAfterUpdateRedirectParams(null)
			);
		}
	}
}
 