<?php

require_once 'abstract.ilPluginControllerGUI.php';


/**
 * Class ilQuestionSetPoolSkillAdministrationGUI
 *
 * @author  Thomas Joußen <tjoussen@databay.de>
 * @since   23.03.15
 *
 * @ilCtrl_Calls ilQuestionSetPoolSkillAdministrationGUI: ilAssQuestionSkillAssignmentsGUI
 * @ilCtrl_Calls ilQuestionSetPoolSkillAdministrationGUI: ilAssQuestionSkillUsagesTableGUI
 */
class ilQuestionSetPoolSkillAdministrationGUI extends ilPLuginControllerGUI
{

	/**
	 * @var ilObjQuestionSetPool
	 */
	protected $object;

	public function __construct(ilObjQuestionSetPoolGUI $controller)
	{
		$this->object = $controller->object;

		parent::__construct($controller);
	}

	public function executeCommand()
	{
		global $DIC;
		$ilDB = $DIC->database();
		$ilPluginAdmin = $DIC['ilPluginAdmin']; 
		$ilAccess = $DIC->access();

		if( $this->isAccessDenied() )
		{
			$this->ilias->raiseError($this->lng->txt("permission_denied"), $this->ilias->error_obj->MESSAGE);
		}

		$nextClass = $this->ctrl->getNextClass();
		$this->setSubTabs($nextClass);
		$questionContainerId = $this->object->getId();

		switch($nextClass)
		{
			case 'ilassquestionskillassignmentsgui':
				$provider = new ilQuestionSetPoolSkillQuestionListProvider();

				$questionList = new ilQuestionSetPoolSkillAssQuestionList($ilDB, $this->lng, $ilPluginAdmin);
				$questionList->setQuestionInstanceTypeFilter($this->getRequiredQuestionInstanceTypeFilter());
				$questionList->setParentObjId($questionContainerId);
				$questionList->setProvider($provider);
				$questionList->load();

				$gui = new ilAssQuestionSkillAssignmentsGUI($this->ctrl, $ilAccess, $this->tpl, $this->lng, $ilDB);
				$gui->setAssignmentEditingEnabled($this->isAssignmentEditingRequired());
				$gui->setQuestionContainerId($questionContainerId);
				$gui->setQuestionList($questionList);
				$gui->setQuestionOrderSequence($questionList->getOrderSequence());
				$this->ctrl->forwardCommand($gui);

				break;
			case 'ilassquestionskillusagestablegui':
				$gui = new ilAssQuestionSkillUsagesTableGUI($this->ctrl, $this->tpl, $this->lng, $ilDB, $this->object->getId());

				$this->ctrl->forwardCommand($gui);

				break;
		}
	}

	private function getRequiredQuestionInstanceTypeFilter()
	{
		return ilAssQuestionList::QUESTION_INSTANCE_TYPE_DUPLICATES;
	}

	protected function isAssignmentEditingRequired()
	{
		return true;
	}

	protected function isAccessDenied()
	{
		if(!$this->object->isSkillServiceEnabled())
		{
			return true;
		}

		if(!ilObjQuestionSetPool::isSkillManagementGloballyActivated())
		{
			return true;
		}

		return false;
	}

	/**
	 * Returns the current tab name
	 * return string
	 */
	protected function getCurrentTab()
	{
		return 'tst_tab_competences';
	}

	private function setSubTabs($controllerClass)
	{
		$link = $this->ctrl->getLinkTargetByClass(
			'ilAssQuestionSkillAssignmentsGUI', ilAssQuestionSkillAssignmentsGUI::CMD_SHOW_SKILL_QUEST_ASSIGNS
		);
		$this->tabs->addSubTab(
			'ilassquestionskillassignmentsgui', $this->lng->txt('qpl_skl_sub_tab_quest_assign'), $link

		);
		$link = $this->ctrl->getLinkTargetByClass(
			'ilAssQuestionSkillUsagesTableGUI', ilAssQuestionSkillUsagesTableGUI::CMD_SHOW
		);
		$this->tabs->addSubTab(
			'ilassquestionskillusagestablegui', $this->lng->txt('qpl_skl_sub_tab_usages'), $link

		);

		$this->tabs->activateSubTab($controllerClass);
	}

	/**
	 * @return array
	 */
	protected function getRequirements()
	{
		return array(
			'utils/class.ilQuestionSetPoolSkillAssQuestionList.php',
			'providers/class.ilQuestionSetPoolSkillQuestionListProvider.php'
		);
	}
}
 