<?php

require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/providers/class.ilParserQuestionProvider.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/ConditionParser.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/CompositeValidator.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/ExpressionNotSupportedByQuestion.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/QuestionNotExist.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/OperatorNotSupportedByExpression.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/UnsupportedExpression.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/UnsupportedOperation.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/AnswerIndexNotExist.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/MissingBracket.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/QuestionNotReachable.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/AnswerValueNotExist.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/ConditionParserException.php";
require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/Parser/classes/Exception/DuplicateElement.php";

/**
 * Class ilQuestionSetPoolConditionValidator
 *
 * Date: 05.12.13
 * Time: 11:45
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
abstract class ilQuestionSetPoolConditionValidator {

	/**
	 * @param string $condition
	 * @param ilQuestionSetPoolNode $node
	 * @param array $data
	 * @param ilPlugin $plugin
	 *
	 * @return bool
	 */
	public static function isValid($condition, $node,$data, $plugin)
	{
		if($condition == "")
		{
			return true;
		}

		try
		{
			$parser    = new ConditionParser();
			$composite = $parser->parse($condition);
			$validator = ilQuestionSetPoolConditionValidator::initializeValidator($node, $data);
			$validator->validate($composite);
		}
		catch (QuestionNotExist $e)
		{
			ilUtil::sendFailure(
				  sprintf($plugin->txt("question_not_exist"), $e->getQuestionIndex()),
					  true
			);

			return false;
		}
		catch (QuestionNotReachable $e)
		{
			ilUtil::sendFailure(
				sprintf($plugin->txt("question_not_reachable"), $e->getQuestionIndex()),
				true
			);

			return false;
		}
		catch (AnswerIndexNotExist $e)
		{
			ilUtil::sendFailure(
				sprintf($plugin->txt("answer_index_not_exist"), $e->getQuestionIndex(), $e->getAnswerIndex()),
				true
			);

			return false;
		}
		catch (ExpressionNotSupportedByQuestion $e)
		{
			ilUtil::sendFailure(
				  sprintf($plugin->txt("expression_not_supported_by_question"), $e->getQuestionIndex(), $e->getExpression()),
					  true
			);

			return false;
		}
		catch (OperatorNotSupportedByExpression $e)
		{
			ilUtil::sendFailure(
				  sprintf($plugin->txt("operator_not_supported_by_expression"), $e->getOperator(), $e->getExpression()),
					  true
			);

			return false;
		}
		catch (UnsupportedExpression $e)
		{
			ilUtil::sendFailure(
				  sprintf($plugin->txt("expression_not_supported"), $e->getExpression()),
					  true
			);

			return false;
		}
		catch (UnsupportedOperation $e)
		{
			ilUtil::sendFailure(
				  sprintf($plugin->txt("operator_not_supported"), $e->getOperator()),
					  true
			);
			return false;
		}
		catch (UnableToParseCondition $e)
		{
			if($e->getCondition() != "")
			{
				$condition = $e->getCondition();
			}
			ilUtil::sendFailure(
				  sprintf($plugin->txt("unable_to_parse_condition"), $condition),
					  true
			);
			return false;
		}
		catch (MissingBracket $e)
		{
			ilUtil::sendFailure(
				sprintf($plugin->txt("missing_bracket"), $e->getBracket()),
				true
			);
			return false;
		}
		catch (AnswerValueNotExist $e)
		{
			ilUtil::sendFailure(
				sprintf($plugin->txt("answer_value_not_exists"), $e->getValue(), $e->getQuestionIndex(), $e->getAnswerIndex()),
				true
			);
			return false;
		}
		catch (ConditionParserException $e)
		{
			ilUtil::sendFailure(
				sprintf($plugin->txt("invalid_statement"), $e->getColumn()),
				true
			);
			return false;
		}
		catch(DuplicateElement $e)
		{
			ilUtil::sendFailure(
				sprintf($plugin->txt("duplicate_element"), $e->getElement()),
				true
			);
			return false;
		}
		return true;
	}

	/**
	 * @param ilQuestionSetPoolNode $node
	 * @oaram array $data
	 *
	 * @return CompositeValidator
	 */
	public static function initializeValidator($node, $data)
	{
		$provider  = new ilParserQuestionProvider($node->getPath()->getSet()->getId());
		$validator = new CompositeValidator($provider, $node, $data);

		return $validator;
	}
}
 