<?php

/**
 * Class ilQuestionSetPoolQuestionsProviderTest
 *
 * Date: 24.09.13
 * Time: 12:33
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilQuestionSetPoolQuestionsProviderTest extends PHPUnit_Framework_TestCase {

	/**
	 * @var ilQuestionSetPoolQuestionsProvider;
	 */
	protected $provider;

	public function setUp(){
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/providers/class.ilQuestionSetPoolQuestionsProvider.php";
		
		global $DIC;
		$ilDB = $DIC->database();

		$ilDB_mock = $this->getMockBuilder("ilDB")->getMock();
		$ilDB_mock
			->expects($this->any())
			->method("quote")
			->with(2)
			->will($this->returnValue(2));

		$ilDB = $ilDB_mock;

		$this->provider = new ilQuestionSetPoolQuestionsProvider();
	}

	public function testGetSelectPart()
	{
		$method = $this->getMethod("getSelectPart");

		$expected = "qpl_questions.*, qpl_qst_type.type_tag, qpl_qst_type.plugin, object_data.title as pool_title";
		$actual = $method->invokeArgs($this->provider, array());

		$this->assertEquals($expected, $actual);
	}

	public function testGetFromPart()
	{
		$method = $this->getMethod("getFromPart");

		$expected = "qpl_questions";
		$actual = $method->invokeArgs($this->provider, array());

		$this->assertEquals($expected, $actual);
	}

	public function testGetJoinPart()
	{
		$method = $this->getMethod("getJoinPart");

		$expected =
			"INNER JOIN qpl_qst_type ON qpl_questions.question_type_fi = qpl_qst_type.question_type_id ".
			"INNER JOIN object_data ON object_data.obj_id = qpl_questions.obj_fi ".
			"LEFT JOIN rep_robj_xqsp_question ON qpl_questions.question_id = rep_robj_xqsp_question.question_fi";

		$actual = $method->invokeArgs($this->provider, array());

		$this->assertEquals($expected, $actual);
	}

	public function testWherePart()
	{
		$method = $this->getMethod("getWherePart");
		$filter = array(
			"xqsp_id" => 2
		);

		$expected = "qpl_questions.original_id IS NULL AND qpl_questions.tstamp > 0 AND rep_robj_xqsp_question.questionsetpool_fi = 2";

		$actual = $method->invokeArgs($this->provider, array($filter));

		$this->assertEquals($expected, $actual);
	}

	public function testGroupByPart()
	{
		$method = $this->getMethod("getGroupByPart");

		$actual = $method->invokeArgs($this->provider, array());

		$this->assertEquals("", $actual);
	}

	public function testHavingPart()
	{
		$method = $this->getMethod("getHavingPart");
		$filter = array();

		$actual = $method->invokeArgs($this->provider, array($filter));

		$this->assertEquals("", $actual);
	}

	public function testOrderByPart()
	{
		$method = $this->getMethod("getOrderByPart");
		$filter = array();

		$actual = $method->invokeArgs($this->provider, array($filter));

		$this->assertEquals("", $actual);
	}

	/**
	 * Get an ReflectionMethod to test private or protected functions
	 *
	 * @param string $method
	 *
	 * @return ReflectionMethod
	 */
	private function getMethod($method)
	{
		$class = new ReflectionClass("ilQuestionSetPoolQuestionsProvider");
		$method = $class->getMethod($method);
		$method->setAccessible(true);

		return $method;
	}
}
